import { ApplicationRef, ComponentFactory, ComponentFactoryResolver, Injector, Type } from '@angular/core';
import { Observable } from 'rxjs';
import { DialogComponent } from './dialog.component';
export interface DialogOptions {
    index?: number;
    autoCloseTimeout?: number;
    closeByClickingOutside?: boolean;
    backdropColor?: string;
    customFactory?: ComponentFactory<{}>;
}
export declare class DialogServiceConfig {
    container: HTMLElement;
}
export declare class DialogService {
    private resolver;
    private applicationRef;
    private injector;
    /**
     * Placeholder of modal dialogs
     */
    private dialogHolderComponent;
    /**
     * HTML container for dialogs
     * type {HTMLElement}
     */
    private container;
    /**
     * @param resolver resolver
     * @param applicationRef applicationRef
     * @param injector injector
     * @param config config
     */
    constructor(resolver: ComponentFactoryResolver, applicationRef: ApplicationRef, injector: Injector, config: DialogServiceConfig);
    /**
     * Adds dialog
     * @return Observable<T1>
     * @param component component
     * @param data data
     * @param options options
     */
    addDialog<T, T1>(component: Type<DialogComponent<T, T1>>, data?: T, options?: DialogOptions): Observable<T1>;
    /**
     * Hides and removes dialog from DOM
     * @param component DialogComponent
     */
    removeDialog(component: DialogComponent<any, any>): void;
    /**
     * Closes all dialogs
     */
    removeAll(): void;
    /**
     * Creates and add to DOM dialog holder component
     * @return DialogHolderComponent
     */
    private createDialogHolder;
}
